/****************************************************************/
/*                                                              */
/*            Advanced Navigation Kinematica API v1.2           */
/*              Copyright 2018, Advanced Navigation             */
/*                                                              */
/****************************************************************/
/*
 * Copyright (C) 2018 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.kinematica.api.json;

import java.io.StringReader;

import javax.json.Json;
import javax.json.JsonObject;
import javax.json.JsonObjectBuilder;
import javax.json.JsonReader;

public class KinematicaFileJSON
{
	public String name;
	public String category;
	public String type;
	public long size; // bytes

	public KinematicaFileJSON(String name, String category, String type, long size)
	{
		this.name     = name;
		this.category = category;
		this.type     = type;
		this.size     = size;
	}

	public static KinematicaFileJSON fromJson(String fileJson)
	{
		JsonReader jsonReader = Json.createReader(new StringReader(fileJson));
		JsonObject logObject  = jsonReader.readObject();
		jsonReader.close();

		String name     = logObject.getString("name");
		String category = logObject.getString("category");
		String type     = logObject.getString("type");
		long size       = logObject.getInt("size");
		return new KinematicaFileJSON(name, category, type, size);
	}

	public static String toJson(KinematicaFileJSON file)
	{
		JsonObjectBuilder builder = Json.createObjectBuilder();
		builder.add("name", file.name);
		builder.add("category", file.category);
		builder.add("type", file.type);
		builder.add("size", file.size);

		JsonObject logObject = builder.build();

		return logObject.toString();
	}
}
