/****************************************************************/
/*                                                              */
/*            Advanced Navigation Kinematica API v1.2           */
/*              Copyright 2018, Advanced Navigation             */
/*                                                              */
/****************************************************************/
/*
 * Copyright (C) 2018 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.kinematica.api.json;

import java.io.StringReader;

import javax.json.Json;
import javax.json.JsonObject;
import javax.json.JsonObjectBuilder;
import javax.json.JsonReader;

public class LogJSON
{
	public String type;
	public String message;
	public long time; // unix timestamp ms

	public LogJSON(String type, String message, long time)
	{
		this.type    = type;
		this.message = message;
		this.time    = time;
	}

	public static LogJSON fromJson(String logJson)
	{
		JsonReader jsonReader = Json.createReader(new StringReader(logJson));
		JsonObject logObject  = jsonReader.readObject();
		jsonReader.close();

		String type    = logObject.getString("type");
		String message = logObject.getString("message");
		long time      = logObject.getJsonNumber("time").longValue();
		return new LogJSON(type, message, time);
	}

	public static String toJson(LogJSON log)
	{
		JsonObjectBuilder builder = Json.createObjectBuilder();
		builder.add("type", log.type);
		builder.add("message", log.message);
		builder.add("time", log.time);

		JsonObject logObject = builder.build();

		return logObject.toString();
	}

}
