/****************************************************************/
/*                                                              */
/*            Advanced Navigation Kinematica API v1.2           */
/*              Copyright 2018, Advanced Navigation             */
/*                                                              */
/****************************************************************/
/*
 * Copyright (C) 2018 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.kinematica.api.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Map.Entry;

import javax.json.Json;
import javax.json.JsonObject;
import javax.json.JsonReader;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.ContentBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

public class Requests
{
	public static String get(String url) throws Exception
	{
		CloseableHttpClient httpclient = HttpClients.createDefault();
		HttpGet httpGet = new HttpGet(url);
		CloseableHttpResponse response = httpclient.execute(httpGet);

		StringBuffer sb = new StringBuffer();
		try
		{
			HttpEntity entity = response.getEntity();
			BufferedReader in = new BufferedReader(new InputStreamReader(entity.getContent()));

			String inputLine;
			while ((inputLine = in.readLine()) != null)
			{
				sb.append(inputLine);
			}

			in.close();
			EntityUtils.consume(entity);
		}
		finally
		{
			response.close();
		}

		return sb.toString();
	}

	public static String postMultipart(String url, PostMultipartData postData) throws Exception
	{
		HttpClient httpclient = HttpClientBuilder.create().build();
		HttpPost httpPost = new HttpPost(url);

		MultipartEntityBuilder builder = MultipartEntityBuilder.create();
		builder.setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
		for (Entry<String, ContentBody> entry : postData.pairs)
		{
			builder.addPart(entry.getKey(), entry.getValue());
		}

		httpPost.setEntity(builder.build());
		HttpResponse response = httpclient.execute(httpPost);

		StringBuffer sb = new StringBuffer();

		HttpEntity entity = response.getEntity();
		BufferedReader in = new BufferedReader(new InputStreamReader(entity.getContent()));

		String inputLine;
		while ((inputLine = in.readLine()) != null)
		{
			sb.append(inputLine);
		}

		in.close();
		EntityUtils.consume(entity);

		return sb.toString();
	}

	public static String postUrlEncoded(String url, PostUrlEncodedData postData) throws Exception
	{
		HttpClient httpclient = HttpClientBuilder.create().build();
		HttpPost httpPost = new HttpPost(url);
		
		httpPost.setEntity(new UrlEncodedFormEntity(postData.pairs, "UTF-8"));
		HttpResponse response = httpclient.execute(httpPost);

		StringBuffer sb = new StringBuffer();

		HttpEntity entity = response.getEntity();
		BufferedReader in = new BufferedReader(new InputStreamReader(entity.getContent()));

		String inputLine;
		while ((inputLine = in.readLine()) != null)
		{
			sb.append(inputLine);
		}

		in.close();
		EntityUtils.consume(entity);

		return sb.toString();
	}

	public static String getDownload(String url, File file) throws Exception
	{
		CloseableHttpClient httpclient = HttpClients.createDefault();
		HttpGet httpGet = new HttpGet(url);
		CloseableHttpResponse response = httpclient.execute(httpGet);

		try
		{
			HttpEntity entity = response.getEntity();

			if (entity.getContentType().getValue().contains("octet-stream"))
			{
				InputStream is = entity.getContent();

				OutputStream os = new FileOutputStream(file);
				IOUtils.copy(is, os);
				os.close();

				EntityUtils.consume(entity);
			}
			else
			{
				BufferedReader in = new BufferedReader(new InputStreamReader(entity.getContent()));
				String inputLine;
				StringBuffer sb = new StringBuffer();

				while ((inputLine = in.readLine()) != null)
				{
					sb.append(inputLine);
				}

				in.close();
				EntityUtils.consume(entity);

				return sb.toString();
			}
		}
		finally
		{
			response.close();
		}

		return "";
	}

	public static JsonObject parseJson(String json)
	{
		JsonReader jsonReader = Json.createReader(new StringReader(json));
		JsonObject object = jsonReader.readObject();
		jsonReader.close();
		return object;
	}

	private static String getUrlParam(String url, String parameter)
	{
		HashMap<String, String> urlMap = new HashMap<String, String>();
		String queryString = StringUtils.substringAfter(url, "?");
		for (String param : queryString.split("&"))
		{
			urlMap.put(StringUtils.substringBefore(param, "="), StringUtils.substringAfter(param, "="));
		}
		return urlMap.get(parameter);
	}
}
